<?php
/*
 Plugin Name: AWPCP XML Sitemap Plugin
 Version: 3.5
 Plugin URI: http://www.awpcp.com
 Description: Adds AWPCP's classified ad entries to XML sitemaps generated by Google XML Sitemaps Generator plugin (http://www.arnebrachhold.de/projects/wordpress-plugins/google-xml-sitemaps-generator/).
 Requires the Google XML Sitemap plugin and AWPCP to be installed and activated.
 Author: D. Rodenbaugh
 Author URI: http://skylineconsult.com

 Additional Credits:
 Supplemental development provided by SimplerComputing.net, Professional Wordpress Development
 */

define('AWPCP_XML_SITEMAP_MODULE', 'Another WordPress Classifieds Plugin - XML Sitemap Module');

define('AWPCP_XML_SITEMAP_MODULE_BASENAME', basename(dirname(__FILE__)));
define('AWPCP_XML_SITEMAP_MODULE_DIR', rtrim( plugin_dir_path( __FILE__ ), '/' ));
define('AWPCP_XML_SITEMAP_MODULE_URL', rtrim( plugin_dir_url( __FILE__ ), '/' ));
define('AWPCP_XML_SITEMAP_MODULE_DB_VERSION', '3.5');
define('AWPCP_XML_SITEMAP_MODULE_REQUIRED_AWPCP_VERSION', '3.5');
define('AWPCP_XML_SITEMAP_MODULE_REQUIRED_GOOGLE_SITEMAP_GENERATOR_VERSION', '3.2.5');

if (!defined('WP_CONTENT_DIR')) {
	define( 'WP_CONTENT_DIR', ABSPATH . 'wp-content' ); // no trailing slash, full paths only - WP_CONTENT_URL is defined further down
}

if (file_exists(WP_CONTENT_DIR."/plugins/google-sitemap-generator/sitemap-core.php")) {
	require_once(WP_CONTENT_DIR."/plugins/google-sitemap-generator/sitemap-core.php");
}


add_action('plugins_loaded', 'awpcp_xml_sitemap_setup', 20);
function awpcp_xml_sitemap_setup() {
	global $awpcp, $awpcp_db_version;

	if ( !$awpcp || !$awpcp->is_up_to_date() || version_compare( $awpcp_db_version, AWPCP_XML_SITEMAP_MODULE_REQUIRED_AWPCP_VERSION ) < 0 ) {
		add_action( 'admin_notices', 'awpcp_xml_sitemap_awpcp_missing_or_outdated' );
		return;
	}

    if ( ! $awpcp->is_compatible_with( 'xml-sitemap', AWPCP_XML_SITEMAP_MODULE_DB_VERSION ) ) {
        add_action( 'admin_notices', 'awpcp_xml_sitemap_not_compatible_notice' );
        return;
    }

	if ( !awpcp_xml_sitemap_is_google_sitemap_generator_updated() ) {
		add_action( 'admin_notices', 'awpcp_xml_sitemap_google_sitemap_generator_outdated' );
	}

	awpcp_load_plugin_textdomain( __FILE__, 'awpcp-xml-sitemap' );

	add_action( 'init', 'awpcp_xml_sitemap_init' );
}


/**
 * @since 2.0.2
 */
function awpcp_xml_sitemap_google_sitemap_generator_outdated() {
	$message = __( 'The AWPCP XML Sitemap module requires Google Sitemap Generator version %s or newer!', 'awpcp-xml-sitemap' );
	$message = sprintf( $message, AWPCP_XML_SITEMAP_MODULE_REQUIRED_GOOGLE_SITEMAP_GENERATOR_VERSION );
    $message = sprintf( '<strong>%s:</strong> %s', __( 'Error', 'awpcp-xml-sitemap' ), $message );
    echo '<div class="error"><p>' . $message . '</p></div>';
}


/**
 * @since 2.0.2
 */
function awpcp_xml_sitemap_awpcp_missing_or_outdated() {
	$message = __( 'The AWPCP XML Sitemap module requires AWPCP version %s or newer!', 'awpcp-xml-sitemap' );
	$message = sprintf( $message, AWPCP_XML_SITEMAP_MODULE_REQUIRED_AWPCP_VERSION );
    $message = sprintf( '<strong>%s:</strong> %s', __( 'Error', 'awpcp-xml-sitemap' ), $message );
    echo '<div class="error"><p>' . $message . '</p></div>';
}


/**
 * @since 3.0.1
 */
function awpcp_xml_sitemap_not_compatible_notice() {
	if ( function_exists( 'awpcp_module_not_compatible_notice' ) ) {
		echo awpcp_module_not_compatible_notice( 'xml-sitemap', AWPCP_XML_SITEMAP_MODULE_DB_VERSION );
	}
}


/**
 * @since 2.0.2
 */
function awpcp_xml_sitemap_is_google_sitemap_generator_installed() {
	return file_exists( WP_CONTENT_DIR . "/plugins/google-sitemap-generator/sitemap-core.php" );
}


/**
 * @since 2.0.2
 */
function awpcp_xml_sitemap_is_google_sitemap_generator_updated() {
	$installed = awpcp_xml_sitemap_is_google_sitemap_generator_installed();

	$updated = false;
	if ( class_exists( 'GoogleSitemapGenerator' ) ) {
		if ( method_exists( 'GoogleSitemapGenerator', 'IsCustomPostTypesSupported' ) ) {
			$updated = true;
		}
	}

	return $installed && $updated;
}


function awpcp_xml_sitemap_init() {
	add_action( 'awpcp_admin_add_submenu_page', 'awpcp_xml_sitemap_admin_menu', 10, 2 );

	// install various actions to happen on demand
	if ( awpcp_xml_sitemap_is_google_sitemap_generator_updated() ) {
		add_action('sm_buildmap', 'awpcp_generate_ad_entries');
		add_action('sm_buildmap', 'awpcp_generate_category_entries');

		add_action('awpcp_edit_ad', 'awpcp_rebuild_sitemap');
		add_action('awpcp-place-ad', 'awpcp_rebuild_sitemap');
		add_action('awpcp_delete_ad', 'awpcp_rebuild_sitemap');
		add_action('awpcp_disable_ad', 'awpcp_rebuild_sitemap');
		add_action('awpcp_approve_ad', 'awpcp_rebuild_sitemap');
	}
}


function awpcp_generate_category_entries() {
	global $wp_rewrite;

	if (!defined('AWPCP_URL')) return; // awpcp plugin not enabled or not loaded

	$wp_rewrite->flush_rules();

	if ( ! GoogleSitemapGeneratorLoader::LoadPlugin() ) return;

	$generatorObject = &GoogleSitemapGenerator::GetInstance();

	if ( $generatorObject != null ) {
		$categories = AWPCP_Category::query( array(
			'where' => 'category_parent_id = 0',
            'orderby' => 'category_order ASC, category_name',
            'order' => 'ASC',
        ) );

		foreach ( $categories as $category ) {
			$category_url = url_browsecategory( $category );
			$generatorObject->AddUrl( $category_url, time(), 'daily', 0.6 );
		}
	}
}


function awpcp_generate_ad_entries() {
	global $wpdb, $wp_rewrite;

	if (!defined('AWPCP_URL')) return; // awpcp plugin not enabled or not loaded

	$tbl_ads = $wpdb->prefix . "awpcp_ads";

	$wp_rewrite->flush_rules();

	if ( ! GoogleSitemapGeneratorLoader::LoadPlugin() ) return;

	$generatorObject = &GoogleSitemapGenerator::GetInstance(); //Please note the "&" sign!

	if ( $generatorObject != null ) {
		$sql = 'SELECT * FROM ' . AWPCP_TABLE_ADS . ' ';
		$sql.= 'WHERE disabled = 0 ORDER BY ad_postdate';

		$ads = $wpdb->get_results($sql, ARRAY_A);

		foreach ($ads as $ad) {
			//$generatorObject->AddUrl(url_showad($ad['ad_id']), date('Y-m-d', time()).' 00:00:00' , "weekly", 0.25);
			$url = url_showad( $ad['ad_id'] );
			$generatorObject->AddUrl($url, time() , "weekly", 0.6);
		}
	}
}


/**
 * Force a sitemap rebuild
 */
function awpcp_rebuild_sitemap() {
	do_action( 'sm_rebuild' );
}


// add_action('admin_menu', 'awpcp_xml_sitemap_admin_menu');
function awpcp_xml_sitemap_admin_menu($slug, $capability='activate_plugins') {
	add_submenu_page($slug, __('AWPCP XML Sitemap Options', 'awpcp-xml-sitemap' ),
		__('XML Sitemap Options', 'awpcp-xml-sitemap' ), $capability,
		__('Sitemap', 'awpcp-xml-sitemap' ), 'awpcp_xml_sitemap_options');
}

//Display the admin menu page
function awpcp_xml_sitemap_options() {
	if (!current_user_can('manage_options'))  {
		wp_die(__('You do not have sufficient permissions to access this page.', 'awpcp-xml-sitemap' ));
	}

	$okToBuild = true;
	if( isset($_POST[ 'awpcp_sitemap_regenerate' ]) && $_POST[ 'awpcp_sitemap_regenerate' ] == 'Y' ) {
		// Put an settings updated message on the screen
		if ( !defined('AWPCP_URL') ) {
			$okToBuild = false;
			echo awpcp_print_error( __( 'AWPCP not installed, cannot generate sitemap.', 'awpcp-xml-sitemap' ) );
		} else if ( !awpcp_xml_sitemap_is_google_sitemap_generator_updated() ) {
			$okToBuild = false;
			echo awpcp_print_error( __( 'Google XML Sitemaps Generator not installed, cannot generate sitemap.', 'awpcp-xml-sitemap' ) );
		} else {
			echo awpcp_print_message( __( 'Sitemap Regenerated.', 'awpcp-xml-sitemap' ) );
			awpcp_rebuild_sitemap();
		}
	}

	/* Now display the settings editing screen */

	echo '<div class="wrap">';

	if ( awpcp() ) {
		$awpcp_status = sprintf( '<span style="color:green">%s</span>', __( 'Installed', 'awpcp-xml-sitemap' ) );
	} else {
		$awpcp_status = sprintf( '<span style="color:red">%s</span>', __( 'NOT INSTALLED', 'awpcp-xml-sitemap' ) );
	}

	if ( awpcp_xml_sitemap_is_google_sitemap_generator_updated() ) {
		$google_sitemap_generator_status = sprintf( '<span style="color:green">%s</span>', __( 'Installed', 'awpcp-xml-sitemap' ) );
	} else if ( awpcp_xml_sitemap_is_google_sitemap_generator_installed() ) {
		$google_sitemap_generator_status = sprintf( '<span style="color:red">%s</span>', __( 'OUTDATED', 'awpcp-xml-sitemap' ) );
	} else {
		$google_sitemap_generator_status = sprintf( '<span style="color:red">%s</span>', __( 'NOT INSTALLED', 'awpcp-xml-sitemap' ) );
	}

	// header

	echo '<h2>' . __( 'AWPCP XML Sitemap Settings', 'awpcp-xml-sitemap' ) . '</h2>';

	echo '<p>';
	echo __( 'The AWPCP XML Sitemap plugin requires the following plugins to be installed:', 'awpcp-xml-sitemap' ) . "<br/>";
	echo sprintf( '<a href="%s">%s</a><br/>', 'http://awpcp.com', __( 'Another Wordpress Classifieds Plugin (AWPCP)', 'awpcp-xml-sitemap' ) );
	echo sprintf( '<a href="%s">%s</a><br/>', 'http://www.arnebrachhold.de/projects/wordpress-plugins/google-xml-sitemaps-generator/', __( 'Google XML Sitemap Generator', 'awpcp-xml-sitemap' ) );
	echo '</p>';

	echo "<h3>" . __( 'Sitemap Installation Status', 'awpcp-xml-sitemap' ) . "</h3>";

	echo '<p>';
	echo sprintf( '<strong>%s (%s or newer): %s</strong><br/>', __( 'AWPCP', 'awpcp-xml-sitemap' ), AWPCP_XML_SITEMAP_MODULE_REQUIRED_AWPCP_VERSION, $awpcp_status );
	echo sprintf( '<strong>%s (%s or newer): %s</strong>', __( 'Google XML Sitemap Generator', 'awpcp-xml-sitemap' ), AWPCP_XML_SITEMAP_MODULE_REQUIRED_GOOGLE_SITEMAP_GENERATOR_VERSION, $google_sitemap_generator_status );
	echo '</p>';

	if ( $okToBuild ) {
		$message = __( "Your sitemap will build automatically as Ads are created/edited/disabled. You don't need to rebuild it unless there is some problem.", 'awpcp-xml-sitemap' );
		echo awpcp_print_message( $message );
		$message = sprintf( __( 'You may view your sitemap here: %s or perhaps here if you set it to GZIP instead: %s', 'awpcp-xml-sitemap' ), '<a target="_blank" href="/sitemap.xml">sitemap.xml</a>', '<a target="_blank" href="/sitemap.xml.gz">sitemap.xml.gz</a>' );
		echo sprintf( '<p>%s</p>', $message );
		$message = __( 'Only one of those links will work, depending on your Google XML Sitemap Generator settings.', 'awpcp-xml-sitemap' );
		echo sprintf( '<p><strong>%s</strong>: %s</p>', __( 'NOTE', 'awpcp-xml-sitemap' ), $message );
	} else {
		echo awpcp_print_error( __( 'PLEASE INSTALL THE MISSING COMPONENTS ABOVE', 'awpcp-xml-sitemap' ) );
	}

?>

<form name="form1" method="post" action="">
	<input type="hidden" name="awpcp_sitemap_regenerate" value="Y">
	<?php if ($okToBuild): ?>
	<p class="submit">
		<input type="submit" name="Submit" class="button-primary" value="<?php _e( 'Rebuild Sitemap Now', 'awpcp-xml-sitemap' ); ?>" />
	</p>
	<?php endif; ?>
</form>

<?php
	echo "</div>";
}
